#include <edapi/journal/entry.h>
#include <edapi/util.h>

#include <jansson.h>

typedef struct {
    json_t *entry;
} EDJournalEntryPrivate;

G_DEFINE_QUARK("ed-json-error", ed_json_error);

G_DEFINE_TYPE_EXTENDED(
    EDJournalEntry,
    ed_journal_entry,
    G_TYPE_OBJECT,
    0,
    G_ADD_PRIVATE(EDJournalEntry)
    );

static void ed_journal_entry_finalize(GObject *obj)
{
    EDJournalEntry *self = ED_JOURNALENTRY(obj);
    EDJournalEntryPrivate *p = ed_journal_entry_get_instance_private(self);

    json_decref(p->entry);
    p = NULL;

    G_OBJECT_CLASS(ed_journal_entry_parent_class)->finalize(obj);
}

static void ed_journal_entry_class_init(EDJournalEntryClass *klass)
{
    G_OBJECT_CLASS(klass)->finalize = ed_journal_entry_finalize;
}

static void ed_journal_entry_init(EDJournalEntry *self)
{
}

EDJournalEntry *ed_journal_entry_new(void)
{
    return g_object_new(
        ED_TYPE_JOURNALENTRY,
        NULL
        );
}

EDJournalEntry *ed_journal_entry_new_parse(gchar const *line, GError **error)
{
    EDJournalEntry *e = ed_journal_entry_new();
    return_if_true(e == NULL, NULL);

    EDErrorCode ret = ed_error_internal;

    ret = ed_journal_entry_parse(e, line, error);
    if (ED_ERROR(ret)) {
        g_clear_object(&e);
        return NULL;
    }

    return e;
}

EDErrorCode ed_journal_entry_parse(EDJournalEntry *self,
                                   gchar const *line,
                                   GError **error)
{
    EDJournalEntryPrivate *p = ed_journal_entry_get_instance_private(self);
    EDErrorCode ret = ed_error_invalid;
    json_error_t e;

    json_decref(p->entry);
    p->entry = NULL;

    memset(&e, 0, sizeof(e));

    p->entry = json_loads(line, 0, &e);
    if (p->entry == NULL) {
        g_set_error(
            error,
            ed_json_error_quark(),
            ed_error_invalid_json,
            "%d: %d: %d: %s",
            e.line,
            e.column,
            e.position,
            e.text
            );
        ret = ed_error_invalid_json;
    } else {
        ret = ed_error_success;
    }

    return ret;
}
